@@ -283,6 +283,49 @@ def test_multiple_global_invariants():
         assert e.invariant_errors == (('x', 'y'),)
 
 
+def test_inherited_global_invariants():
+    class Distant(object):
+        def __invariant__(self):
+            return [(self.distant, "distant")]
+
+    class Nearby(Distant):
+        def __invariant__(self):
+            return [(self.nearby, "nearby")]
+
+    class MultipleInvariantGlobal(Nearby, PClass):
+        distant = field()
+        nearby = field()
+
+    try:
+        MultipleInvariantGlobal(distant=False, nearby=False)
+        assert False
+    except InvariantException as e:
+        assert e.invariant_errors == (("nearby",), ("distant",),)
+
+
+def test_diamond_inherited_global_invariants():
+    counter = []
+    class Base(object):
+        def __invariant__(self):
+            counter.append(None)
+            return [(False, "base")]
+
+    class Left(Base):
+        pass
+
+    class Right(Base):
+        pass
+
+    class SingleInvariantGlobal(Left, Right, PClass):
+        pass
+
+    try:
+        SingleInvariantGlobal()
+        assert False
+    except InvariantException as e:
+        assert e.invariant_errors == (("base",),)
+        assert counter == [None]
+
 def test_supports_weakref():
     import weakref
     weakref.ref(Point(x=1, y=2))
@@ -357,4 +400,4 @@ def test_enum_key_type():
     class MyClass2(PClass):
         f = pmap_field(key_type=(Foo,), value_type=int)
 
-    MyClass2()
\ No newline at end of file
+    MyClass2()
